use AdventureWorks2008R2
go
drop table #orders
go

Select SalesPersonID,
       min(OrderDate) as OrderMonth,
       sum(TotalDue ) as TotalDue
into  #Orders
  from Sales.SalesOrderHeader SOH
where SOH.SalesPersonID is not null
group by SalesPersonID,datediff(mm,0,OrderDate);

go
create clustered index idxOrder on #Orders(SalesPersonId,OrderMonth);
go

select * from #Orders
order by SalesPersonID,OrderMonth;

go
/* Triangular join Rolling Balance */
select *,
       (select sum(TotalDue)
	     from  #Orders Balance
		  where Balance.SalesPersonId = #Orders.SalesPersonID
		   and  Balance.OrderMonth <=   #Orders.OrderMonth) as RollingBalance
       from #Orders
order by SalesPersonID,OrderMonth;
go
/* 2005 + , Sum grouping by a partition of data */
select *,
       sum(TotalDue) over(partition by SalesPersonId) as SalesPersonTotal
  from #Orders
order by SalesPersonID,OrderMonth
go

/* Denali, Support for order by in over clause for aggregates */
select *,
       sum(TotalDue) over(partition by SalesPersonId order by OrderMonth)
	      as RollingBalance
  from #Orders
order by SalesPersonID,OrderMonth;

go
/* Default is Range UNBOUNDED PRECEDING */
select *,
       sum(TotalDue) over(partition by SalesPersonId order by OrderMonth)
	      as RollingBalanceDefault,
       sum(TotalDue) over(partition by SalesPersonId order by OrderMonth 
	      Range UNBOUNDED PRECEDING) as RollingBalance
  from #Orders
order by SalesPersonID,OrderMonth;
go
/* Have support for ROWS */
select *,
       sum(TotalDue) over(partition by SalesPersonId order by OrderMonth 
	      Rows UNBOUNDED PRECEDING) as RollingBalance
  from #Orders
order by SalesPersonID,OrderMonth;
go

/* Difference in Row and Range,  is ....??? */
Drop table #simpleSales
go

Create Table #SimpleSales
(
   SaleID integer not null,
   LineId integer not null,
   Sales  money   not null)
go

insert into #SimpleSales
values
(1,1,1),(1,2,5),(1,2,10),(1,3,20),
(2,1,101),(2,2,105),(2,2,110),(2,3,120)

go
Select * 
  from #SimpleSales 
 Order by SaleId,LineId
go

/* Range Operates summing the balance for each distinct set of values
of <Partition> and <Order by> columns.
Rows operates on a row-by-row basis, regardless of value */

select * ,
       sum(Sales) over (partition by SaleId order by LineId 
	                    Range unbounded Preceding) as UnboundedRange,
	   sum(Sales) over (partition by SaleId order by LineId 
	                    Rows  unbounded Preceding) as UnboundedRows
  from #SimpleSales
order by SaleId, LineId
go

/* Lets see that in our data */
begin transaction
insert into #Orders(SalesPersonID,OrderMonth,TotalDue)
select SalesPersonId,OrderMonth,TotalDue
 from  #Orders
where SalesPersonID = 274
 and  DATEPART(yy,OrderMonth)=2006

 select *,
       sum(TotalDue) over(partition by SalesPersonId order by OrderMonth 
	                  Range UNBOUNDED PRECEDING) as RangeUnbound,
       sum(TotalDue) over(partition by SalesPersonId order by OrderMonth
	                  Rows UNBOUNDED PRECEDING)  as RowUnbound
  from #Orders
order by SalesPersonID,OrderMonth


rollback


go
/* Compare Range and Rows over a large number of rows */

Drop Table #LargeOrders
go
Create Table #LargeOrders
(
SalesPersonId integer not null,
OrderMonth datetime not null,
TotalDue   money not null)

go
Create unique clustered index idxLargeOrders on #LargeOrders
(
SalesPersonId,OrderMonth
)
go
insert into #LargeOrders
Select SalesPersonID,OrderMonth,sum(TotalDue)
  from #Orders
group  by SalesPersonId,OrderMonth

go
Declare @MaxSalesID integer
Select top (1) @MaxSalesID = SalesPersonId
  from #LargeOrders
  order by SalesPersonId desc

insert into #LargeOrders
Select SalesPersonID+@MaxSalesID,OrderMonth,TotalDue
  from #LargeOrders

go 9

select count(*) from #LargeOrders
go

drop table #1
go
drop table #2
go
declare @T1 datetime  = getdate()
 select *,
       sum(TotalDue) over(partition by SalesPersonId 
	     order by OrderMonth range Unbounded Preceding)  as SumTotalDue
 into #1
  from #LargeOrders
  
select 'Range Unbounded Completed', datediff(ms,@t1,getdate())

go
declare @T1 datetime  = getdate()
select *,
       sum(TotalDue) over(partition by SalesPersonId 
	    order by OrderMonth Rows UNBOUNDED PRECEDING)  as SumTotalDue
  into #2
from #LargeOrders

select 'Rows Unbounded Completed', datediff(ms,@t1,getdate())
go





/* Use of between clause */
select *,
       sum(TotalDue)   over(partition by SalesPersonId order by OrderMonth 
	       Rows between 4 PRECEDING and current row)  as SumTotalDue,
	   min(OrderMonth) over(partition by SalesPersonId order by OrderMonth 
	       Rows between 4 PRECEDING and current row)  as MinOrderMonth,
	   max(OrderMonth) over(partition by SalesPersonId order by OrderMonth 
	       Rows between 4 PRECEDING and current row)  as MaxOrderMonth
from #Orders
where SalesPersonID = 274
order by OrderMonth
go

/* Opposite of preceding is following */
select *,
       sum(TotalDue)   over(partition by SalesPersonId order by OrderMonth 
	          Rows between current row and 4 following )  as SumTotalDue,
	   avg(TotalDue)   over(partition by SalesPersonId order by OrderMonth 
	          Rows between current row and 4 following )  as AvgTotalDue,
	   max(OrderMonth) over(partition by SalesPersonId order by OrderMonth 
	          Rows between current row and 4 following )  as MaxOrderMonth
from #Orders
where SalesPersonID = 274
order by OrderMonth


go
/*  Remember we are talking ROWS , not data in context */
/*  Many Months have no sales , what does that mean ? */

select *,
       sum(TotalDue)   over(order by DateRange.Month 
	       Rows between current row and 4 following )  as SumTotalDue,
	   avg(TotalDue)   over(order by DateRange.Month 
	       Rows between current row and 4 following )  as AvgTotalDue,
	   max(OrderMonth) over(order by DateRange.Month 
	       Rows between current row and 4 following )  as MaxOrderMonth
from #Orders right outer join ( select dateadd(mm,number,'2005-08-01')
                      from master..spt_values 
					  where Type='P'
					 and number <20) as DateRange(Month)
		 on #Orders.OrderMonth = DateRange.Month
and SalesPersonID = 274
order by DateRange.Month

go

/* Sliding Window of Ranges */
Create table #RangeTest
(
SalesPersonId integer,
SalesId       integer,
LineId        integer,
Val           money
)
go

insert into #RangeTest
values (1,1,1,1),(1,1,2,1),(1,1,3,1),
       (1,2,1,1),(1,2,2,1),(1,2,3,1),
	   (1,3,1,1),(1,3,2,1),(1,3,3,1),
	   (1,4,1,1),(1,4,2,1),(1,4,3,1)
go
Select *,
       sum(val) over(partition by SalesPersonId order by SalesId 
	        Range between 2 preceding and current row)
  from #RangeTest


