use AdventureWorks2008R2
go
drop table #orders
go

Select SalesPersonID,
       min(OrderDate) as OrderMonth,
       sum(TotalDue ) as TotalDue
into  #Orders
  from Sales.SalesOrderHeader SOH
where SOH.SalesPersonID is not null
group by SalesPersonID,datediff(mm,0,OrderDate);

go
create unique clustered index idxOrder on #Orders(SalesPersonId,OrderMonth);
go

select * from #Orders
order by SalesPersonID,OrderMonth;


go
/* Looking at Lag , Returns Value from X many rows behind*/
Select * ,
       LAG(OrderMonth)  OVER (Partition by SalesPersonId 
	                          Order by OrderMonth) as Lag1Month
  from #Orders
order by SalesPersonID,OrderMonth;

go

/* With no offset ,1 is default */
Select * ,
       LAG(OrderMonth,1)  OVER (Partition by SalesPersonId 
	                          Order by OrderMonth) as Lag1Month
  from #Orders
order by SalesPersonID,OrderMonth;

go
/* Default value */
Select * ,
       LAG(OrderMonth,1)  OVER (Partition by SalesPersonId Order by OrderMonth) as Lag1Month,
       LAG(OrderMonth,1,'2011-01-01')  OVER (Partition by SalesPersonId Order by OrderMonth) as Lag1MonthDefault
  from #Orders
order by SalesPersonID,OrderMonth;
go

/* Show Last order months difference */
Select * ,
       LAG(OrderMonth,1)  OVER (Partition by SalesPersonId 
	             Order by OrderMonth) as LastOrderMonth,
       datediff(mm,LAG(OrderMonth,1)  OVER (Partition by SalesPersonId 
	            Order by OrderMonth),OrderMonth ) as MonthOrderDiff
  from #Orders
order by SalesPersonID,OrderMonth;
go

/* Compare to 2005+ Method */
Select #Orders.*,
       LastOrderMonth.OrderMonth,
	   datediff(mm,LastOrderMonth.OrderMonth,#Orders.OrderMonth ) as MonthOrderDiff
 from #Orders
 outer apply(Select top(1) InnerOrders.OrderMonth
              from  #Orders InnerOrders
			  where InnerOrders.SalesPersonID = #Orders.SalesPersonID
			    and InnerOrders.OrderMonth <  #Orders.OrderMonth
			order by InnerOrders.OrderMonth desc) as LastOrderMonth
go

/* Create more data */

Declare @MaxSalesID integer
Select top (1) @MaxSalesID = SalesPersonId
  from #Orders
  order by SalesPersonId desc

insert into #Orders
Select SalesPersonID+@MaxSalesID,OrderMonth,TotalDue
  from #Orders

go 9 
select count(*) from #Orders
go

drop table #3
drop table #2
go
declare @T1 datetime = getdate()
/* Show Last order months difference */
Select * ,
       LAG(OrderMonth,1)  OVER (Partition by SalesPersonId Order by OrderMonth) as LastOrderMonth,
       datediff(mm,LAG(OrderMonth,1)  OVER (Partition by SalesPersonId Order by OrderMonth),OrderMonth ) as MonthOrderDiff
	   into #3
  from #Orders
order by SalesPersonID,OrderMonth;

select 'Denali Lag',datediff(ms,@t1,getdate())

go
declare @T1 datetime = getdate()
/* Compare to 2005+ Method */
Select #Orders.*,
       LastOrderMonth.OrderMonth as LastOrderMonth,
	   datediff(mm,LastOrderMonth.OrderMonth,#Orders.OrderMonth ) as MonthOrderDiff
	into #2
 from #Orders
 outer apply(Select top(1) InnerOrders.OrderMonth
              from  #Orders InnerOrders
			  where InnerOrders.SalesPersonID = #Orders.SalesPersonID
			    and InnerOrders.OrderMonth <  #Orders.OrderMonth
			order by InnerOrders.OrderMonth desc) as LastOrderMonth
select '2005+ Lag',datediff(ms,@t1,getdate())
go

drop table #2
drop table #3


go

/*   LAG default is 1 row behind but can be X rows */
Select * ,
       LAG(OrderMonth,1)  OVER (Partition by SalesPersonId Order by OrderMonth) as Lag1Month,
	   LAG(OrderMonth,2)  OVER (Partition by SalesPersonId Order by OrderMonth) as Lag2Month
  from #Orders
  where  SalesPersonID = 274
order by SalesPersonID,OrderMonth;
go

Select * ,
       LAG(OrderMonth,1)  OVER (Partition by SalesPersonId Order by OrderMonth) as Lag1Month,
	   LAG(TotalDue,1)  OVER (Partition by SalesPersonId Order by OrderMonth) as Lag1TotalDue
  from #Orders
  where  SalesPersonID = 274
order by SalesPersonID,OrderMonth;


go
declare @i integer
Select @i = 3
Select * ,
       LAG(OrderMonth,@i)  OVER (Partition by SalesPersonId Order by OrderMonth) as LagiMonth
  from #Orders
where  SalesPersonID = 274
order by SalesPersonID,OrderMonth;


go
/* Scalar Expressions returned */

Select * ,
       LAG(DatePart(mm,OrderMonth),1)  OVER (Partition by SalesPersonId Order by OrderMonth) as LagMonth,
	   LAG(DatePart(yy,OrderMonth),1)  OVER (Partition by SalesPersonId Order by OrderMonth) as LagYear
  from #Orders
where  SalesPersonID = 274
order by SalesPersonID,OrderMonth;




/* Rows / Range not supported */
Select * ,
       LAG(OrderMonth,1)  OVER (Partition by SalesPersonId
	         Order by OrderMonth ROWS UNBOUNDED PRECEDING) as LagYear
  from #Orders
where  SalesPersonID = 274
order by SalesPersonID,OrderMonth;

go


/* LEAD -- Opposite of lag */

Select * ,
       LEAD(OrderMonth,1)  OVER (Partition by SalesPersonId Order by OrderMonth) as Lead1Month
  from #Orders
  where  SalesPersonID = 274
order by SalesPersonID,OrderMonth;


/* First_value */
go
Select * 
 from  #Orders
 cross apply( Select top(1) innerOrders.TotalDue
                     from   #Orders innerOrders
					 where  innerOrders.SalesPersonID = #Orders.SalesPersonID
					 order by innerOrders.OrderMonth) as FirstOrderValue

  order by SalesPersonID,OrderMonth
go

select *,
       first_value(Totaldue) over( partition by SalesPersonId 
	                              order by OrderMonth)
  from #Orders
    order by SalesPersonID,OrderMonth
	
  go
/* First_Value can window */
 select *,
       first_value(Totaldue) over( order by OrderMonth ROWS  5 PRECEDING)
  from #Orders
 where  SalesPersonID = 274
  order by SalesPersonId,OrderMonth,TotalDue
go

  select *,
       first_value(Totaldue) over( order by OrderMonth ROWS  
	           between 3 following and 5 Following)
  from #Orders
 where  SalesPersonID = 274
  order by SalesPersonId,OrderMonth,TotalDue

go
/* Opposite of first_value is last_value */
select *,
       last_value(Totaldue) over( order by OrderMonth)
  from #Orders
    where  SalesPersonID = 274
  order by SalesPersonID,OrderMonth


go
drop table #2
drop table #3
go
declare @t1 datetime = getdate()

Select #Orders.*, 
       FirstOrderValue.TotalDue as FirstTotDue
 into #2
 from  #Orders
 cross apply( Select top(1) innerOrders.TotalDue
                     from   #Orders innerOrders
					 where  innerOrders.SalesPersonID = #Orders.SalesPersonID
					 order by innerOrders.OrderMonth) as FirstOrderValue

  order by SalesPersonID,OrderMonth

select datediff(ms,@t1,getdate()) as 'CrossApply Time'

go
declare @t1 datetime = getdate()

select *,
       first_value(Totaldue) over( partition by SalesPersonId 
	                              order by OrderMonth) as FirstTotDue
	   into #3
  from #Orders
    order by SalesPersonID,OrderMonth

select datediff(ms,@t1,getdate()) as 'FirstValue Time' 

go

select * from #2 
except 
select * from #3

go
select * from #3
except 
select * from #2
